/* Copyright (c) 1999 Carlo Wood.  All rights reserved.
 * Copyright (c) 1994 Joseph Arceneaux.  All rights reserved.
 * Copyright (c) 1992 Free Software Foundation, Inc.  All rights reserved.
 *
 * Copyright (c) 1985 Sun Microsystems, Inc. Copyright (c) 1980 The Regents
 * of the University of California. Copyright (c) 1976 Board of Trustees of
 * the University of Illinois. All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that
 * the above copyright notice and this paragraph are duplicated in all such
 * forms and that any documentation, advertising materials, and other
 * materials related to such distribution and use acknowledge that the
 * software was developed by the University of California, Berkeley, the
 * University of Illinois, Urbana, and Sun Microsystems, Inc.  The name of
 * either University or Sun Microsystems may not be used to endorse or
 * promote products derived from this software without specific prior written
 * permission. THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES
 * OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * 2002-11-10 Cristalle Azundris Sabon <cristalle@azundris.com>
 *			  Added --preprocessor-indentation (ppi)   if set, will indent nested
 *			  preprocessor-statements with n spaces per level.	overrides -lps.
 */

#ifndef INDENT_INDENT_H
#define INDENT_INDENT_H

#define USE_BADS_BAPS     /*+++ use -badsN -bapN options */
#include "sys.h"

RCSTAG_H(indent, "$Id: indent.h,v 1.37 2002/11/10 21:02:48 david Exp $");



#include "config.h"

#include "libgettext.h"

#ifdef ENABLE_NLS
 #define _(X) 		gettext(X)
#else
#ifdef HAVE_TINY_GETTEXT
 #define _(X) 		gettext(X)
#else
 #define _(X) 		X
#endif
#endif

#include "lexi.h"
#include "io.h"



/* Values that `indent' can return for exit status.
 *
 *	`total_success' means no errors or warnings were found during a successful
 *	  invocation of the program.
 *
 *	`invocation_error' is returned if an invocation problem (like an incorrect
 *	  option) prevents any formatting to occur.
 *
 *	`indent_error' is returned if errors occur during formatting which
 *	  do not prevent completion of the formatting, but which appear to be
 *	  manifested by incorrect code (i.e, code which wouldn't compile).
 *
 *	`indent_punt' is returned if formatting of a file is halted because of
 *	  an error with the file which prevents completion of formatting.  If more
 *	  than one input file was specified, indent continues to the next file.
 *
 *	`indent_fatal' is returned if a serious internal problem occurs and
 *	  the entire indent process is terminated, even if all specified files
 *	  have not been processed. */

typedef enum exit_values {
	total_success		= 0,
	invocation_error	= 1,
	indent_error		= 2,
	indent_punt 		= 3,
	indent_fatal		= 4,
	system_error		= 5
} exit_values_ty;


typedef unsigned char	BOOLEAN;

#define false 			0
#define true  			1

#define DEFAULT_RIGHT_MARGIN				78
#define DEFAULT_RIGHT_COMMENT_MARGIN		78


/* Name of input file.	*/
extern char *		in_name;

extern char *		in_prog;		/* pointer to the null-terminated input program */

/* Point to the position in the input program which we are currently looking at. */
extern char *		in_prog_pos;

/* Point to the start of the current line.	*/
extern char *		cur_line;


/* Size of the input program, not including the ' \n\0' we add at the end */
extern unsigned long in_prog_size;


/* The output file. */

extern char *		labbuf;						/* buffer for label */
extern char *		s_lab; 						/* start ... */
extern char *		e_lab; 						/* .. and end of stored label */
extern char *		l_lab; 						/* limit of label buffer */

extern char *		codebuf;					/* buffer for code section */
extern char *		s_code;						/* start ... */
extern char *		s_code_corresponds_to; 		/* If set, start of corresponding code in token buffer... */
extern char *		e_code;						/* .. and end of stored code */
extern char *		l_code;						/* limit of code section */

extern char *		combuf;						/* buffer for comments */
extern char *		s_com; 						/* start ... */
extern char *		e_com; 						/* ... and end of stored comments */
extern char *		l_com; 						/* limit of comment buffer */

extern char *		buf_ptr;					/* ptr to next character to be taken from in_buffer */
extern char *		buf_end;					/* ptr to first after last char in in_buffer */

extern int			break_line; 				/* Whether or not we should break the line. */



/* pointer to the token that lexi() has just found */
extern char *		token;

/* points to the first char after the end of token */
extern char *		token_end;

extern int				squest;
extern file_buffer_ty * current_input;


#include "save_com.h"


extern char *	bp_save;							/* saved value of buf_ptr when taking input from save_com */
extern char *	be_save;							/* similarly saved value of buf_end */

extern int		pointer_as_binop;
extern int		n_real_blanklines;
extern int		prefix_blankline_requested;
extern codes_ty prefix_blankline_requested_code;	/* The code that caused the blank line to be requested */
extern int		postfix_blankline_requested;
extern codes_ty postfix_blankline_requested_code;	/* The code that caused the blank line to be requested */
extern int		break_comma;						/* when true and not in parens, break after a comma */


#include "settings.h"


/* True if there is an embedded comment on this code line */
extern int		embedded_comment_on_line;

extern int		else_or_endif;
extern int		di_stack_alloc;
extern int		*di_stack;

extern int		else_or_endif;					/* True if a #else or #endif has been encountered.	*/
extern int		code_lines; 					/* count of lines with code */
extern int		out_lines;						/* the number of lines written, set by dump_line */
extern int		com_lines;						/* the number of lines with comments, set by dump_line */
extern BOOLEAN	had_eof;						/* set to true when input is exhausted */
extern int		line_no;						/* the current input line number. */

extern int					output_line_length(void);


#include "parse.h"



#endif	/* INDENT_INDENT_H */
