/* Copyright (c) 1999 Carlo Wood.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef INDENT_PARSE_H
#define INDENT_PARSE_H

//RCSTAG_H (parse, "$Id: parse.h,v 1.6 2002/02/26 20:38:27 david Exp $");

//#include "indent.h"


/* This structure contains information relating to the state of parsing the code.
   The difference is that the state is saved on #if and restored on #else.
 */

typedef struct parser_state {
	struct parser_state *	next;

	codes_ty	last_token;

	/* This is the parsers stack, and the current allocated size.  */
	codes_ty *	p_stack;
	int 		p_stack_size;

	/* This stack stores indentation levels Currently allocated size is stored in p_stack_size.  */
	int *		il;

	/* If the last token was an ident and is a reserved word, remember the type. */
	rwcodes_ty	last_rw;

	/* also, remember its depth in parentheses */
	int 		last_rw_depth;

	/* Used to store case stmt indentation levels.	*/
	/* Currently allocated size is stored in p_stack_size.	*/
	int *		cstk;

	/* Pointer to the top of stack of the p_stack, il and cstk arrays. */
	int 		tos;

	int 		box_com;					/* set to true when we are in a "boxed" comment.
											   In that case, the first non-blank char should be
											   lined up with the / in the comment closing delimiter */

	int 		cast_mask;					/* indicates which close parens close off casts */

	/* A bit for each paren level, set if the open paren was in a context which
	   indicates that this pair of parentheses is not a cast.  */
	int 		noncast_mask;

	int 		sizeof_mask;				/* indicates which close parens close off sizeof''s */
	int 		block_init; 				/* set to 1 if inside a block initialization
											   set to 2 if inside an enum declaration */
	int 		block_init_level;			/* The level of brace nesting in an initialization (0 in an enum decl) */
	int 		last_nl;					/* this is true if the last thing scanned was a newline */
	int 		last_saw_nl;				/* this is true if the last non white space scanned was a newline */
	int 		saw_double_colon;			/* set when we see a ::, reset at first semi-colon or left brace */
	int 		broken_at_non_nl;			/* true when a line was broken at a place
											   where there was no newline in the input file */
	int 		in_or_st;					/* Will be true iff there has been a declarator (e.g. int or char)
											   and no left paren since the last semicolon. When true,
											   a '{' is starting a structure definition or an initialization list */
	int 		col_1;						/* set to true if the last token started in column 1 */
	int 		com_col;					/* this is the column in which the current coment should start */
	int 		dec_nest;					/* current nesting level for structure or init */
	int 		decl_on_line;				/* set to true if this line of code has part of a declaration on it */
	int 		i_l_follow; 				/* the level in spaces to which ind_level should be set
											   after the current line is printed */
	BOOLEAN 	in_decl;					/* set to true when we are in a declaration statement.
											   The processing of braces is then slightly different */
	int 		in_stmt;					/* set to 1 while in a stmt */
	int 		in_parameter_declaration;
	int 		ind_level;					/* the current indentation level in spaces */
	int 		ind_stmt;					/* set to 1 if next line should have an extra indentation level
											   because we are in the middle of a stmt */
	int 		last_u_d;					/* set to true after scanning a token
											   which forces a following operator to be unary */
	int 		p_l_follow; 				/* used to remember how to indent following statement */
	int 		paren_level;				/* parenthesization level. used to indent within stmts */
	int 		paren_depth;				/* Depth of paren nesting anywhere. */

	/* Column positions of paren at each level.
	   If positive, it contains just the number of characters of code on the line up to
	   and including the right parenthesis character. If negative,
	   it contains the opposite of the actual level of indentation in characters
	   (that is, the indentation of the line has been added to the number of characters
		and the sign has been reversed to indicate that this has been done).
	 */
	short *		paren_indents; 				/* column positions of each paren */
	int 		paren_indents_size; 		/* Currently allocated size.  */

	int 		pcase;						/* set to 1 if the current line label is a case.
											   It is printed differently from a regular label */
	int 		search_brace;				/* set to true by parse when it is necessary to buffer up
											   all info up to the start of a stmt after an if, while, etc */
	int 		use_ff; 					/* set to one if the current line should be terminated with a form feed */
	int 		want_blank; 				/* set to true when the following token should be prefixed by a blank.
											   (Said prefixing is ignored in some cases.) */
	bb_code_ty	can_break;					/* set when a break is ok before the following token
											   (is automatically implied by `want_blank'. */
	int 		its_a_keyword;
	int 		sizeof_keyword;
	char *		procname;					/* The name of the current procedure */
	char *		procname_end;				/* One char past the last one in procname */
	char *		classname; 					/* The name of the current C++ class */
	char *		classname_end; 				/* One char past the last one in classname */
	int 		just_saw_decl;
	int 		matching_brace_on_same_line; /* Set to a value >= 0 if the the current '}'
												has a matching '{' on the same input line */
} parser_state_ty;



/* All manipulations of the parser state occur at the top of stack (tos).
   A stack is kept for conditional compilation (unrelated to the p_stack, il,
   & cstk stacks)--it is implemented as a linked list via the next field.
 */
extern parser_state_ty*		parser_state_tos;


extern int				inc_pstack (void);
extern void 			parse_lparen_in_decl (void);
extern exit_values_ty	parse (codes_ty tk);
extern void 			init_parser (void);
extern void 			reset_parser (void);
extern void 			reduce (void);

#endif /* INDENT_PARSE_H */
