/* Copyright (c) 1999 Carlo Wood.  All rights reserved.
 * Copyright (c) 1994 Joseph Arceneaux.  All rights reserved.
 * Copyright (c) 1992 Free Software Foundation, Inc.  All rights reserved.
 *
 * Copyright (c) 1985 Sun Microsystems, Inc. Copyright (c) 1980 The Regents
 * of the University of California. Copyright (c) 1976 Board of Trustees of
 * the University of Illinois. All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that
 * the above copyright notice and this paragraph are duplicated in all such
 * forms and that any documentation, advertising materials, and other
 * materials related to such distribution and use acknowledge that the
 * software was developed by the University of California, Berkeley, the
 * University of Illinois, Urbana, and Sun Microsystems, Inc.  The name of
 * either University or Sun Microsystems may not be used to endorse or
 * promote products derived from this software without specific prior written
 * permission. THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES
 * OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * 2002-11-10 Cristalle Azundris Sabon <cristalle@azundris.com>
 *			  Added --preprocessor-indentation (ppi)   if set, will indent nested
 *			  preprocessor-statements with n spaces per level.	overrides -lps.
 */

#ifndef INDENT_SAVE_COM_H
#define INDENT_SAVE_COM_H


/* Used to keep track of buffers.  */
typedef struct save_com_ty {
	char *		ptr;							/* points to the start of the buffer */
	char *		end;							/* points to the character beyond the last one
												   (e.g. is equal to ptr if the buffer is empty). */
	int 		size;							/* how many chars are currently allocated.	*/
	int 		len;							/* how many chars we're actually using. */
	int 		start_column;					/* corresponding column of first character in the buffer. */
	int 		column; 						/* Column we were in when we switched buffers. */
} save_com_ty;



/* Buffer in which to save a comment which occurs between an if(), while(),
 * etc., and the statement following it.  Note: the fact that we point into
 * this buffer, and that we might realloc() it (via the need_chars macro) is
 * a bad thing (since when the buffer is realloc'd its address might change,
 * making any pointers into it point to garbage), but since the filling of
 * the buffer (hence the need_chars) and the using of the buffer (where
 * buf_ptr points into it) occur at different times, we can get away with it
 * (it would not be trivial to fix).  */

extern save_com_ty	save_com;


#endif
