/* Copyright (c) 1999 Carlo Wood.  All rights reserved.
 * Copyright (c) 1994 Joseph Arceneaux.  All rights reserved.
 * Copyright (c) 1992 Free Software Foundation, Inc.  All rights reserved.
 *
 * Copyright (c) 1985 Sun Microsystems, Inc. Copyright (c) 1980 The Regents
 * of the University of California. Copyright (c) 1976 Board of Trustees of
 * the University of Illinois. All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that
 * the above copyright notice and this paragraph are duplicated in all such
 * forms and that any documentation, advertising materials, and other
 * materials related to such distribution and use acknowledge that the
 * software was developed by the University of California, Berkeley, the
 * University of Illinois, Urbana, and Sun Microsystems, Inc.  The name of
 * either University or Sun Microsystems may not be used to endorse or
 * promote products derived from this software without specific prior written
 * permission. THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES
 * OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * 2002-11-10 Cristalle Azundris Sabon <cristalle@azundris.com>
 *			  Added --preprocessor-indentation (ppi)   if set, will indent nested
 *			  preprocessor-statements with n spaces per level.	overrides -lps.
 */

#ifndef INDENT_SETTINGS_H
#define INDNET_SETTINGS_H


/*
 * This structure stores all the user options that come from (e.g.)
 * command line flags.
 */
typedef struct user_options_st {
	int 		verbose;						/* when true, non-essential error messages are printed */
	int 		use_tabs;						/* does indent convert tabs to spaces or not */
	int 		tabsize;						/* The number of columns a tab character generates. */
	int 		use_stdout; 					/* Where does output go. */
	int 		space_sp_semicolon; 			/* If true, a space is inserted between if, while, or for,
												   and a semicolon for example while (*p++ == ' ') ; */
	int 		swallow_optional_blanklines;
	int 		star_comment_cont;				/* true if comment continuation lines should have stars
												   at the beginning of each line. */
	int 		struct_brace_indent;			/* Indentation level to be used for a '{'
												   directly following a struct, union or enum */
	int 		space_after_while;				/* Put a space after `while' */
	int 		space_after_if; 				/* Put a space after `if' */
	int 		space_after_for;				/* Put a space after `for' */
	int 		procnames_start_line;			/* if true, the names of procedures being defined get placed in column 1
												   (ie. a newline is placed between the type of the procedure and its name) */
	int 		parentheses_space;				/* If true, parentheses will look like: ( foo ) rather than (foo) */
	int 		preserve_mtime; 				/* True when the modification time of the files should	be preserved. */
	int 		paren_indent;					/* set to the indentation per open parens */
	int 		proc_calls_space;				/* If true, procedure calls look like: foo (bar) rather than foo(bar) */
	int 		leave_preproc_space;			/* if true, leave the spaces between  '#' and preprocessor commands. */
	int 		force_preproc_width;			/* if this is >0, this becomes the preproc indent-level */
	int 		lineup_to_parens;				/* if true, continued code within parens will be lined up to the open paren */
	int 		honour_newlines;				/* True when positions at which we read a newline in the input file,
												   should get a high priority to break long lines at. */
	int 		format_comments;				/* If any comments are to be reformatted */
	int 		format_col1_comments;			/* If comments which start in column 1 are to be magically reformatted */
	int 		extra_expression_indent;		/* True if continuation lines from the expression part of "if(e)",
												 * "while(e)", "for(e;e;e)" should be indented an extra tab stop so that
												 * they don't conflict with the code that follows */
	int 		ljust_decl; 					/* true if declarations should be left justified */
	int 		cast_space; 					/* If true, casts look like: (char *) bar rather than (char *)bar */
	int 		cuddle_else;					/* true if else should cuddle up to '}' */
	int 		cuddle_do_while;				/* true if '}' should cuddle up to while in do loop */
	int 		comment_delimiter_on_blankline;
	int 		blank_after_sizeof; 			/* true iff a blank should always be inserted after sizeof */
	int 		break_function_decl_args;		/* true if declarations should have args on new lines */
	int 		break_function_decl_args_end;	/* true if declarations should have ")" after args on new lines */
	int 		leave_comma;					/* if true, never break declarations after commas */
	int 		break_before_boolean_operator;	/* True when we prefer to break a long line before a '&&' or '||',
												   instead of behind it. */
	int 		blanklines_before_blockcomments;
	int 		blanklines_after_declarations;
  #ifdef USE_BADS_BAPS							/*+++ for -badsN -bapN */
	int 		blanklines_after_declarations_n;
	int 		blanklines_after_procs_n;
  #endif
	int 		blanklines_after_procs;
	int 		blanklines_after_declarations_at_proctop;	/* This is vaguely similar to blanklines_after_declarations
															   except that it only applies to the first set of
															   declarations in a procedure (just after the first '{')
															   and it causes a blank line to be generated even if there
															   are no declarations */
	int 		blanklines_around_conditional_compilation;
	int 		comment_max_col;
	int 		max_col;						/* the maximum allowable line length */
	int 		ind_size;						/* The size of one indentation level in spaces.  */
	int 		indent_parameters;				/* Number of spaces to indent parameters.  */
	int 		decl_indent;					/* column to indent declared identifiers to */
	int 		unindent_displace;				/* comments not to the right of code will be placed
												   this many indentation levels to the left of code */
	int 		else_endif_col; 				/* The column in which comments to
												   the right of #else and #endif should start. */
	int 		case_indent;					/* The distance to indent case labels from the switch statement */
	int 		continuation_indent;			/* set to the indentation between the edge of code and continuation lines in spaces */
	int 		decl_com_ind;					/* the column in which comments after declarations should be put */
	int 		case_brace_indent;				/* Indentation level to be used for a '{'
												 * directly following a case label. */
	int 		c_plus_plus;					/* True if we're handling C++ code. */
	int 		com_ind;						/* the column in which comments to the right of code should start */
	int 		braces_on_struct_decl_line; 	/* when true, brace should be on same line as the struct declaration */
	int 		braces_on_func_def_line;		/* when true, brace should be on same line as the function definition */
	int 		btype_2;						/* when true, brace should be on same line as if, while, etc */
	int 		brace_indent;					/* number of spaces to indent braces from the suround if, while, etc. in -bl
												 * (bype_2 == 0) code */
	int 		expect_output_file; 			/* Means "-o" was specified. */
  #ifdef USE_MBC
	int 		use_mbc;						/* use multi-bytes-char */
  #endif
} user_options_ty;



extern user_options_ty	settings;



#endif
