/**
 *  @file   tiny_gettext.c
 *  @brief  fake gettext() for Windows
 *  @author tenk@6809.net
 */

#include <stdio.h>
#include <assert.h>
#include "config.h"
#include "libgettext.h"

#ifdef HAVE_TINY_GETTEXT

static char *   msgEngConv_data[] = {
    #include "tiny_gettext.inc"
};




/* -------------------------------------------------------------------------*/

enum { MSG_ENGCONV_NUM = sizeof(msgEngConv_data)/sizeof(msgEngConv_data[0]) / 2 - 1};
static char *   msgEngConvTbl[MSG_ENGCONV_NUM][2];
static int      msgEngConvTbl_num;
static int      msgEngConv_flag = 1;



/* -------------------------------------------------------------------------*/

static int msgEngConvTbl_add(char *tbl[][2], int *tblcnt, char **key);
static int msgEngConvTbl_search(char *tbl[][2], int nn, const char *key);


/* -------------------------------------------------------------------------*/
#ifdef WIN32
# include <windows.h>
#endif


void textdomain(const char * dmy_domainName)
{
    int i;

  #ifdef WIN32
    unsigned    msg_locale_id = strtoul(msgEngConv_data[1], 0, 0);
    msgEngConv_flag           = (GetThreadLocale() == msg_locale_id);
  #else
    msgEngConv_flag           = 1;
  #endif
    if (msgEngConv_flag) {
        msgEngConvTbl_num = 0;
        for (i = 0; i < MSG_ENGCONV_NUM; i++) {
            msgEngConvTbl_add(&msgEngConvTbl[0], &msgEngConvTbl_num, &msgEngConv_data[2+i*2]);
        }
    }
}



char *  gettext(const char *  msgid)
{
    if (msgEngConvTbl_num > 0) {
        int  n;
        n = msgEngConvTbl_search(msgEngConvTbl, msgEngConvTbl_num, msgid);
        if (n >= 0)
            return  msgEngConvTbl[n][1];
    }
    return (char *)msgid;
}



static int msgEngConvTbl_add(char* tbl[][2], int* tblcnt, char** key)
{
    int  low, mid, f, hi;

    hi  = *tblcnt;
    mid = low = 0;
    while (low < hi) {
        mid = (low + hi - 1) / 2;
        f = strcmp(key[0], tbl[mid][0]);
        if (f < 0) {
            hi = mid;
        } else if (f > 0) {
            mid++;
            low = mid;
        } else {
            assert(0);
            return -1;
        }
    }
    (*tblcnt)++;
    for (hi = *tblcnt; --hi > mid;) {
        tbl[hi][0] = tbl[hi-1][0];
        tbl[hi][1] = tbl[hi-1][1];
    }
    tbl[mid][0] = key[0];
    tbl[mid][1] = key[1];
    return 0;
}



static int msgEngConvTbl_search(char* tbl[][2], int nn, const char* key)
{
    int     low, mid, f;

    low = 0;
    while (low < nn) {
        mid = (low + nn - 1) / 2;
        f = strcmp(key, tbl[mid][0]);
        if (f < 0)
            nn = mid;
        else if (f > 0)
            low = mid + 1;
        else
            return mid;
    }
    return -1;
}


#endif
